--[[notsoeasygametype - flexible configuration for gametypes
	based on easygametype v0.3.2 by goulart and fafabis
	edit by lonsfor
]]

local ezgt_servername = CV_RegisterVar({
	name = "ezgt_servername",
	defaultvalue = "",
	flags = CV_NETVAR
})

local escape = {
	{a = "\\128", b = "\128"},
	{a = "\\129", b = "\129"},
	{a = "\\130", b = "\130"},
	{a = "\\131", b = "\131"},
	{a = "\\132", b = "\132"},
	{a = "\\133", b = "\133"},
	{a = "\\134", b = "\134"},
	{a = "\\135", b = "\135"},
	{a = "\\136", b = "\136"},
	{a = "\\137", b = "\137"},
	{a = "\\138", b = "\138"},
	{a = "\\139", b = "\139"},
	{a = "\\140", b = "\140"},
	{a = "\\141", b = "\141"},
	{a = "\\142", b = "\142"},
	{a = "\\143", b = "\143"}
}
local function doEscapeCode(str)
	for _, v in pairs(escape)
		str = str:gsub(v.a, v.b)
	end
	return str
end

local ezgt_debug = CV_RegisterVar({
	name = "ezgt_debug",
	defaultvalue = "Off",
	PossibleValue = CV_OnOff
})

local ezgt_roundmax = CV_RegisterVar({
	name = "ezgt_roundmax",
	defaultvalue = "5",
	PossibleValue = {MIN = 1, MAX = 20},
	flags = CV_NETVAR
})


local gametypes = {}
local nextgameindex = nil
local currentgameindex = nil
local loaded = false
local roundsleft = 0
local roundstoplay = 2

local chatstring


local function isvotable(command) -- check if command is votable
	if server.HMvotable and type(server.HMvotable) == "table" then 
		for _, votable in pairs(server.HMvotable) do
			if votable == command then
				return true
			end
		end
		return false
	end
end


COM_AddCommand("addgametype", function(player, ...) -- add gametypes, recomend to use wait command in between aditions to prevent the buffer overflow thing(like you should do woth hostmod :^))
	if player ~= server then
		CONS_Printf(player, "This is the Host's job")
		return
	end

	if ... == nil then
		CONS_Printf(player, "Add a gametype to the manager\nFormat example:\nname=Name,shortname=Shortname,on=var1 val|var2 val,off=var1 val|var2 val") --format example
		return
	end

	local newgame = table.concat({...}, " ")
	if ezgt_debug.value == 1 then
		CONS_Printf(player, newgame)
	end

	local gametypetable = {name="", shortname="", on={}, off={}}

	for key, valthing in string.gmatch(newgame, "([^,=]+)=([^,=]+)") do

		if key == "name" then
			gametypetable.name = valthing

		elseif key == "shortname" then
			gametypetable.shortname = valthing

		elseif key == "on" then
			for command, setting in string.gmatch(valthing, "([^| ]+) ([^| ]+)") do
				if not CV_FindVar(command) then -- make sure the command exists
					CONS_Printf(player, "The command ".. command .." was not found, Make sure it's wad/pk3 is added and try again.")
					return
				end

				table.insert(gametypetable.on, {variable=command, value=setting})
			end

		elseif key == "off" then
			for command, setting in string.gmatch(valthing, "([^| ]+) ([^|]+)") do
				if not CV_FindVar(command) then
					CONS_Printf(player, "The command ".. command .." was not found, Make sure it's wad/pk3 is added and try again.")
					return
				end

				table.insert(gametypetable.off, {variable=command, value=setting})
			end
		end

	end
	
	if gametypetable.name == "" then
		CONS_Printf(player, "You didn't give the gametype its name")
		return
	end
	if #gametypetable.on == 0 then
		CONS_Printf(player, "You didn't give the gametype its On settings")
		return
	end
	if #gametypetable.off == 0 then
		CONS_Printf(player, "You didn't give the gametype its Off settings")
		return
	end

	table.insert(gametypes, gametypetable)

	if ezgt_debug.value then
		local addmessege = "Ready: ".. gametypetable.name
		if gametypetable.shortname ~= "" then
			addmessege = $+ "(".. gametypetable.shortname ..")"
		end

	
		addmessege = $+ "\nOn settings: "
		for _, b in pairs(gametypetable.on) do
			addmessege = $+ b.variable .." \""+ b.value+"\", "
		end
		addmessege = $+ "\nOff settings: "
		for _, b in pairs(gametypetable.off) do
			offsettings = $+ b.variable +" \"" +b.value+"\", "
		end

		CONS_Printf(player, addmessege)
	end
end, 1)


COM_AddCommand("resetgametype", function(player, nukecode)
	if currentgameindex == nil and nextgameindex == nil and nukecode == nil then
		CONS_Printf(player, "No gametype currently running")

	elseif (player == server or IsPlayerAdmin(player)) then
		roundsleft = 0
		nextgameindex = nil
		loaded = false -- if in intermition/votetime load again

		if nukecode ~= nil and nukecode == "full" then -- the nuclear option. use in case of emergency.
			currentgameindex = "nuke"

			chatprint("\131*FULL RESET INITIETED")
			COM_BufInsertText(player, "wait 35; exitlevel")
		else
			chatprint("\131*Gametype will be reset next round", true)
		end

	else
		CONS_Printf(player,"You dont have permition to reset the gametype")
		if isvotable(player, "resetgametype") then
			CONS_Printf(player,"Try \130vote resetgametype \128instead")
		end
	end
end)


COM_AddCommand("setgametype", function(player, askedgametype, askedrounds)
	if askedgametype == nil and askedrounds == nil then
		local infostring = "Gametypes avelable: \n\n"
		if #gametypes ~= 0 then
			for _, namepair in pairs(gametypes) do
				infostring = $+ namepair.name
				if namepair.shortname ~= "" then
					infostring = $+ " (".. namepair.shortname ..")"
				end
				infostring = $+ "\n"
			end

			infostring = $+ "\nUse \130setgametype [gametype] \133[#rounds]\n                            *optional\n\130extendgametype \133[#rounds]\n\130resetgametype\128\n"

			if currentgameindex ~= nil then
				infostring = $+ "\nCurrent Gametype: ".. gametypes[currentgameindex].name .." ("..roundsleft.." rounds)"
			end
			if nextgameindex ~= nil then
				infostring = $+ "\nQueued Gametype: ".. gametypes[nextgameindex].name .." ("..roundstoplay.." rounds)"
			end

		else
			infostring = $+ "none :("
		end

		CONS_Printf(player, infostring)

	elseif (player == server or IsPlayerAdmin(player)) then
		if askedgametype == "" then return end -- this should never happen but you never know

		if nextgameindex ~= nil then
			if isvotable("setgametype") then
				chatprint("\131*A gametype is already queued.", true)
			else
				CONS_Printf(player, "A gametype is already queued.")
			end

		elseif currentgameindex ~= nil and roundsleft >= 1 then
			if isvotable("setgametype") then
				chatprint("\131*A gametype is already running, wait for it to finish.", true)
			else
				CONS_Printf(player, "A gametype is already running, wait for it to finish.")
			end

		else
			if ezgt_debug.value == 1 then
				CONS_Printf(player, "DEBUG: searching for ".. askedgametype)
				if askedrounds ~= nil then
					CONS_Printf(player, askedrounds.." rounds")
				end
			end

			askedgametype = askedgametype:lower()

			for gametypeindex, namepair in pairs(gametypes) do --look for the names/shortnames of the gametypes
				if askedgametype == namepair.name:lower() or askedgametype == namepair.shortname:lower() then
					if type(gametypeindex) ~= "number" then
						CONS_Printf(server, "OH NO some fuckup shit happened")
					else
						nextgameindex = gametypeindex

						if ezgt_debug.value == 1 then
							CONS_Printf(server, "DEBUG: nextgameindex set to ".. nextgameindex)
						end

						askedrounds = tonumber(askedrounds)
						if askedrounds ~= nil and r > 0 then
							roundstoplay = min(ezgt_roundmax.value, askedrounds)
						else
							roundstoplay = 2
						end

						loaded = false --in case setgametype happens during intermission/vote

						chatstring = "\131*Next gametype will be \130".. namepair.name .. "\131 for \130".. roundstoplay
						if roundstoplay == 1 then
							chatstring = $+ " round"
						else
							chatstring = $+ " rounds"
						end
						chatprint(chatstring, true)
						chatstring = nil

						return
					end

				end

			end
			if isvotable("setgametype") then -- no way to check if a command was done with vote so maybe this will do
				chatprint("\131*No gametype by the name. Check your spelling?", true)
			else
				CONS_Printf(player, "No gametype by that name. Check your spelling?")
			end
		end

	else
		CONS_Printf(player,"You dont have permition to set a gametype")
		if isvotable("setgametype") then
			CONS_Printf(player,"Try \130vote setgametype \128instead")
		end
	end
end)
COM_AddCommand("listgametypes", function(player)
	COM_BufInsertText(player, "setgametype")
end)


COM_AddCommand("extendgametype", function(player, roundnum)
	roundnum = tonumber(roundnum)
	if roundnum == nil or roundnum <= 0 then
		CONS_Printf(player, "Extend the ammount of rounds to play the current gametype")
	else
		if (player == server or IsPlayerAdmin(player)) then
			if currentgameindex == nil then
				CONS_Printf(player, "No gametype currently running")
			else
				roundsleft = min(ezgt_roundmax.value, roundsleft + roundnum)

				chatstring = "\131*Gametype extended to \130".. roundsleft
				if roundsleft == 1 then
					chatstring = $+ " round"
				else
					chatstring = $+ " rounds"
				end
				chatprint(chatstring, true)
				chatstring = nil
			end
		else
			CONS_Printf(player,"You dont have permition to extend the gametype")
			if isvotable("extendgametype") then
				CONS_Printf(player,"Try \130vote extendgametype \128instead")
			end
		end
	end
end)


addHook("MapLoad", function()
	loaded = false
	nextgameindex = nil

	if currentgameindex ~= nil then
		if roundsleft == 1 then
			COM_BufInsertText(server, "wait 35; csay ".. gametypes[currentgameindex].name .."\\Last Round")
		else
			COM_BufInsertText(server, "wait 35; csay ".. gametypes[currentgameindex].name .."\\".. tostring(roundsleft) .." Rounds Left")
		end
	end

	roundsleft = max($-1, 0)
	
	if mapheaderinfo[gamemap].menuflags & LF2_HIDEINMENU then -- its a hell map
		for player in players.iterate do
			if player.valid then
				if player == server and ((player.mo == nil and player.name == "Player 1") or player.spectator) then --is host dedicated(maybe)/spec
					continue
				else
					return --a player was found, do nothing.
				end
			end
		end
		COM_BufAddText(server, "map " .. G_BuildMapName(0))
	end
end)

local function loadnexgame()
	if loaded == true then return end

	if roundsleft == 0 and currentgameindex ~= nil then --unload gametype
		if currentgameindex == "nuke" then --nuclear option
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "DEBUG: NUCEAR OPTION ENGAGED")
			end
			for _, gametypetable in pairs(gametypes) do
				for _, varvalpair in pairs(gametypetable.off) do --run through every gametype and set it off
					COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
					if ezgt_debug.value == 1 then
						CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
					end
				end
			end
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "")
			end
		else --normal unload
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "DEBUG: unloading ".. gametypes[currentgameindex].name)
			end
			for _, varvalpair in pairs(gametypes[currentgameindex].off) do
				COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
				if ezgt_debug.value == 1 then
					CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
				end
			end
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "")
			end
		end
		currentgameindex = nil

		if ezgt_servername.string ~= "" then --turn back server name
			COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..'"')
			if ezgt_debug.value == 1 then
				CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..'"')
			end
		end

		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: unload done")
		end
	end

	if nextgameindex ~= nil then --load new gametype
		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: loading ".. gametypes[nextgameindex].name)
		end
		for _, varvalpair in pairs(gametypes[nextgameindex].on) do
			COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
			if ezgt_debug.value == 1 then
				CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
			end
		end
		currentgameindex = nextgameindex
		roundsleft = roundstoplay

		if ezgt_servername.string ~= "" then --change server name
			if gametypes[nextgameindex].shortname ~= "" then --if gametype doesnt have a shortname use the normal name
				COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].shortname ..'"')
				if ezgt_debug.value == 1 then
					CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].shortname ..'"')
				end
			else
				COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].name ..'"')
				if ezgt_debug.value == 1 then
					CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].name ..'"')
				end
			end
		end

		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: load done")
		end
	end

	loaded = true
	if ezgt_debug.value == 1 then
		CONS_Printf(server, "DEBUG: next game ready")
	end
end
addHook("IntermissionThinker", loadnexgame)
addHook("VoteThinker", loadnexgame)


addHook("PlayerQuit", function(quiter) --reset game when no players are in the server
	if currentgameindex or nextgameindex then -- a gametype is on/queued
		for player in players.iterate do
			if player.valid then
				if player == quiter then
					continue
				elseif player == server and ((player.mo == nil and player.name == "Player 1") or player.spectator) then --is host dedicated(maybe)/spec
					continue
				else
					return --a player was found, do nothing.
				end
			end
		end
		CONS_Printf(server, "No players. resetting gametype")
		COM_BufInsertText(server, "resetGametype")
		COM_BufInsertText(server, "exitlevel")
	elseif mapheaderinfo[gamemap].menuflags & LF2_HIDEINMENU then -- its a hell map
		for player in players.iterate do
			if player == quiter then
				continue
			elseif player == server and ((player.mo == nil and player.name == "Player 1") or player.spectator) then
				continue
			else
				return --a player was found, do nothing.
			end
		end
		COM_BufInsertText(server, "map " .. G_BuildMapName(0))
	end
end)


addHook("NetVars", function(n)
	gametypes = n($)
	currentgameindex = n($)
	nextgameindex = n($)
	loaded = n($)
	roundsleft = n($)
	roundstoplay = n($)
end)
